@@ -71,18 +71,19 @@ struct ev_loop;
 #endif
 
 /* eventmask, revents, events... */
-#define EV_UNDEF          -1 /* guaranteed to be invalid */
-#define EV_NONE         0x00
-#define EV_READ         0x01 /* io only */
-#define EV_WRITE        0x02 /* io only */
-#define EV_TIMEOUT  0x000100 /* timer only */
-#define EV_PERIODIC 0x000200 /* periodic timer only */
-#define EV_SIGNAL   0x000400 /* signal only */
-#define EV_IDLE     0x000800 /* idle only */
-#define EV_CHECK    0x001000 /* check only */
-#define EV_PREPARE  0x002000 /* prepare only */
-#define EV_CHILD    0x004000 /* child/pid only */
-#define EV_ERROR    0x800000 /* sent when an error occurs */
+#define EV_UNDEF          -1L /* guaranteed to be invalid */
+#define EV_NONE         0x00L 
+#define EV_READ         0x01L /* io only */
+#define EV_WRITE        0x02L /* io only */
+#define EV_TIMEOUT  0x000100L /* timer only */
+#define EV_PERIODIC 0x000200L /* periodic timer only */
+#define EV_SIGNAL   0x000400L /* signal only */
+#define EV_IDLE     0x000800L /* idle only */
+#define EV_CHECK    0x001000L /* check only */
+#define EV_PREPARE  0x002000L /* prepare only */
+#define EV_CHILD    0x004000L /* child/pid only */
+#define EV_EMBED    0x008000L /* embedded event loop */
+#define EV_ERROR    0x800000L /* sent when an error occurs */
 
 /* can be used to add custom fields to all watchers, while losing binary compatibility */
 #ifndef EV_COMMON
@@ -126,94 +127,106 @@ struct ev_loop;
   ev_tstamp at;     /* private */
 
 /* base class, nothing to see here unless you subclass */
-struct ev_watcher
+typedef struct ev_watcher
 {
   EV_WATCHER (ev_watcher)
-};
+} ev_watcher;
 
 /* base class, nothing to see here unless you subclass */
-struct ev_watcher_list
+typedef struct ev_watcher_list
 {
   EV_WATCHER_LIST (ev_watcher_list)
-};
+} ev_watcher_list;
 
 /* base class, nothing to see here unless you subclass */
-struct ev_watcher_time
+typedef struct ev_watcher_time
 {
   EV_WATCHER_TIME (ev_watcher_time)
-};
+} ev_watcher_time;
 
 /* invoked after a specific time, repeatable (based on monotonic clock) */
 /* revent EV_TIMEOUT */
-struct ev_timer
+typedef struct ev_timer
 {
   EV_WATCHER_TIME (ev_timer)
 
   ev_tstamp repeat; /* rw */
-};
+} ev_timer;
 
 /* invoked at some specific time, possibly repeating at regular intervals (based on UTC) */
 /* revent EV_PERIODIC */
-struct ev_periodic
+typedef struct ev_periodic
 {
   EV_WATCHER_TIME (ev_periodic)
 
   ev_tstamp interval; /* rw */
   ev_tstamp (*reschedule_cb)(struct ev_periodic *w, ev_tstamp now); /* rw */
-};
+} ev_periodic;
 
 /* invoked when fd is either EV_READable or EV_WRITEable */
 /* revent EV_READ, EV_WRITE */
-struct ev_io
+typedef struct ev_io
 {
   EV_WATCHER_LIST (ev_io)
 
   int fd;     /* ro */
   int events; /* ro */
-};
+} ev_io;
 
 /* invoked when the given signal has been received */
 /* revent EV_SIGNAL */
-struct ev_signal
+typedef struct ev_signal
 {
   EV_WATCHER_LIST (ev_signal)
 
   int signum; /* ro */
-};
+} ev_signal;
 
 /* invoked when the nothing else needs to be done, keeps the process from blocking */
 /* revent EV_IDLE */
-struct ev_idle
+typedef struct ev_idle
 {
   EV_WATCHER (ev_idle)
-};
+} ev_idle;
 
 /* invoked for each run of the mainloop, just before the blocking call */
 /* you can still change events in any way you like */
 /* revent EV_PREPARE */
-struct ev_prepare
+typedef struct ev_prepare
 {
   EV_WATCHER (ev_prepare)
-};
+} ev_prepare;
 
 /* invoked for each run of the mainloop, just after the blocking call */
 /* revent EV_CHECK */
-struct ev_check
+typedef struct ev_check
 {
   EV_WATCHER (ev_check)
-};
+} ev_check;
 
 /* invoked when sigchld is received and waitpid indicates the given pid */
 /* revent EV_CHILD */
 /* does not support priorities */
-struct ev_child
+typedef struct ev_child
 {
   EV_WATCHER_LIST (ev_child)
 
   int pid;     /* ro */
   int rpid;    /* rw, holds the received pid */
   int rstatus; /* rw, holds the exit status, use the macros from sys/wait.h */
-};
+} ev_child;
+
+#if EV_MULTIPLICITY
+/* used to embed an event loop inside another */
+/* the callback gets invoked when the event loop has handled events, and can be 0 */
+typedef struct ev_embed
+{
+  EV_WATCHER (ev_embed)
+
+  ev_io io; /* private */
+  struct ev_loop *loop; /* ro */
+} ev_embed;
+#endif
 
 /* the presence of this union forces similar struct layout */
 union ev_any_watcher
@@ -229,27 +242,32 @@ union ev_any_watcher
   struct ev_check check;
   struct ev_signal signal;
   struct ev_child child;
+#if EV_MULTIPLICITY
+  struct ev_embed embed;
+#endif
 };
 
 /* bits for ev_default_loop and ev_loop_new */
 /* the default */
-#define EVFLAG_AUTO      0x00000000 /* not quite a mask */
-
-/* method bits to be ored together */
-#define EVMETHOD_SELECT  0x00000001 /* about anywhere */
-#define EVMETHOD_POLL    0x00000002 /* !win */
-#define EVMETHOD_EPOLL   0x00000004 /* linux */
-#define EVMETHOD_KQUEUE  0x00000008 /* bsd */
-#define EVMETHOD_DEVPOLL 0x00000010 /* solaris 8 */ /* NYI */
-#define EVMETHOD_PORT    0x00000020 /* solaris 10 */
-
+#define EVFLAG_AUTO       0x00000000UL /* not quite a mask */
 /* flag bits */
-#define EVFLAG_NOENV     0x01000000 /* do NOT consult environment */
+#define EVFLAG_NOENV      0x01000000UL /* do NOT consult environment */
+/* method bits to be ored together */
+#define EVBACKEND_SELECT  0x00000001UL /* about anywhere */
+#define EVBACKEND_POLL    0x00000002UL /* !win */
+#define EVBACKEND_EPOLL   0x00000004UL /* linux */
+#define EVBACKEND_KQUEUE  0x00000008UL /* bsd */
+#define EVBACKEND_DEVPOLL 0x00000010UL /* solaris 8 */ /* NYI */
+#define EVBACKEND_PORT    0x00000020UL /* solaris 10 */
 
 #if EV_PROTOTYPES
 int ev_version_major (void);
 int ev_version_minor (void);
 
+unsigned int ev_supported_backends (void);
+unsigned int ev_recommended_backends (void);
+unsigned int ev_embeddable_backends (void);
+
 ev_tstamp ev_time (void);
 
 /* Sets the allocation function to use, works like realloc.
@@ -308,11 +326,12 @@ void ev_default_destroy (void); /* destroy the default loop */
 /* you can actually call it at any time, anywhere :) */
 void ev_default_fork (void);
 
-unsigned int ev_method (EV_P);
+unsigned int ev_backend (EV_P);
 #endif
 
 #define EVLOOP_NONBLOCK	1 /* do not block/wait */
 #define EVLOOP_ONESHOT	2 /* block *once* only */
+#define EVUNLOOP_CANCEL 0 /* undo unloop */
 #define EVUNLOOP_ONE    1 /* unloop once */
 #define EVUNLOOP_ALL    2 /* unloop all loops */
 
@@ -336,9 +355,9 @@ void ev_once (EV_P_ int fd, int events, ev_tstamp timeout, void (*cb)(int revent
 /* these may evaluate ev multiple times, and the other arguments at most once */
 /* either use ev_init + ev_TYPE_set, or the ev_TYPE_init macro, below, to first initialise a watcher */
 #define ev_init(ev,cb_) do {				\
-  ((struct ev_watcher *)(void *)(ev))->active   =	\
-  ((struct ev_watcher *)(void *)(ev))->pending  =	\
-  ((struct ev_watcher *)(void *)(ev))->priority = 0;	\
+  ((ev_watcher *)(void *)(ev))->active   =	\
+  ((ev_watcher *)(void *)(ev))->pending  =	\
+  ((ev_watcher *)(void *)(ev))->priority = 0;	\
   ev_set_cb ((ev), cb_);					\
 } while (0)
 
@@ -350,6 +369,7 @@ void ev_once (EV_P_ int fd, int events, ev_tstamp timeout, void (*cb)(int revent
 #define ev_prepare_set(ev)                  /* nop, yes, this is a serious in-joke */
 #define ev_check_set(ev)                    /* nop, yes, this is a serious in-joke */
 #define ev_child_set(ev,pid_)               do { (ev)->pid = (pid_); } while (0)
+#define ev_embed_set(ev,loop_)              do { (ev)->loop = (loop_); } while (0)
 
 #define ev_io_init(ev,cb,fd,events)         do { ev_init ((ev), (cb)); ev_io_set ((ev),(fd),(events)); } while (0)
 #define ev_timer_init(ev,cb,after,repeat)   do { ev_init ((ev), (cb)); ev_timer_set ((ev),(after),(repeat)); } while (0)
@@ -359,11 +379,12 @@ void ev_once (EV_P_ int fd, int events, ev_tstamp timeout, void (*cb)(int revent
 #define ev_prepare_init(ev,cb)              do { ev_init ((ev), (cb)); ev_prepare_set ((ev)); } while (0)
 #define ev_check_init(ev,cb)                do { ev_init ((ev), (cb)); ev_check_set ((ev)); } while (0)
 #define ev_child_init(ev,cb,pid)            do { ev_init ((ev), (cb)); ev_child_set ((ev),(pid)); } while (0)
+#define ev_embed_init(ev,cb,loop)           do { ev_init ((ev), (cb)); ev_embed_set ((ev),(loop)); } while (0)
 
-#define ev_is_pending(ev)                   (0 + ((struct ev_watcher *)(void *)(ev))->pending) /* ro, true when watcher is waiting for callback invocation */
-#define ev_is_active(ev)                    (0 + ((struct ev_watcher *)(void *)(ev))->active) /* ro, true when the watcher has been started */
+#define ev_is_pending(ev)                   (0 + ((ev_watcher *)(void *)(ev))->pending) /* ro, true when watcher is waiting for callback invocation */
+#define ev_is_active(ev)                    (0 + ((ev_watcher *)(void *)(ev))->active) /* ro, true when the watcher has been started */
 
-#define ev_priority(ev)                     ((struct ev_watcher *)(void *)(ev))->priority /* rw */
+#define ev_priority(ev)                     ((ev_watcher *)(void *)(ev))->priority /* rw */
 #define ev_cb(ev)                           (ev)->cb /* rw */
 #define ev_set_priority(ev,pri)             ev_priority (ev) = (pri)
 
@@ -381,36 +402,44 @@ void ev_feed_event     (EV_P_ void *w, int revents);
 void ev_feed_fd_event  (EV_P_ int fd, int revents);
 void ev_feed_signal_event (EV_P_ int signum);
 
-void ev_io_start       (EV_P_ struct ev_io *w);
-void ev_io_stop        (EV_P_ struct ev_io *w);
+void ev_io_start       (EV_P_ ev_io *w);
+void ev_io_stop        (EV_P_ ev_io *w);
 
-void ev_timer_start    (EV_P_ struct ev_timer *w);
-void ev_timer_stop     (EV_P_ struct ev_timer *w);
+void ev_timer_start    (EV_P_ ev_timer *w);
+void ev_timer_stop     (EV_P_ ev_timer *w);
 /* stops if active and no repeat, restarts if active and repeating, starts if inactive and repeating */
-void ev_timer_again    (EV_P_ struct ev_timer *w);
+void ev_timer_again    (EV_P_ ev_timer *w);
 
 #if EV_PERIODICS
-void ev_periodic_start (EV_P_ struct ev_periodic *w);
-void ev_periodic_stop  (EV_P_ struct ev_periodic *w);
-void ev_periodic_again (EV_P_ struct ev_periodic *w);
+void ev_periodic_start (EV_P_ ev_periodic *w);
+void ev_periodic_stop  (EV_P_ ev_periodic *w);
+void ev_periodic_again (EV_P_ ev_periodic *w);
 #endif
 
-void ev_idle_start     (EV_P_ struct ev_idle *w);
-void ev_idle_stop      (EV_P_ struct ev_idle *w);
+void ev_idle_start     (EV_P_ ev_idle *w);
+void ev_idle_stop      (EV_P_ ev_idle *w);
 
-void ev_prepare_start  (EV_P_ struct ev_prepare *w);
-void ev_prepare_stop   (EV_P_ struct ev_prepare *w);
+void ev_prepare_start  (EV_P_ ev_prepare *w);
+void ev_prepare_stop   (EV_P_ ev_prepare *w);
 
-void ev_check_start    (EV_P_ struct ev_check *w);
-void ev_check_stop     (EV_P_ struct ev_check *w);
+void ev_check_start    (EV_P_ ev_check *w);
+void ev_check_stop     (EV_P_ ev_check *w);
 
 /* only supported in the default loop */
-void ev_signal_start   (EV_P_ struct ev_signal *w);
-void ev_signal_stop    (EV_P_ struct ev_signal *w);
+void ev_signal_start   (EV_P_ ev_signal *w);
+void ev_signal_stop    (EV_P_ ev_signal *w);
 
 /* only supported in the default loop */
-void ev_child_start    (EV_P_ struct ev_child *w);
-void ev_child_stop     (EV_P_ struct ev_child *w);
+void ev_child_start    (EV_P_ ev_child *w);
+void ev_child_stop     (EV_P_ ev_child *w);
+
+# if EV_MULTIPLICITY
+/* only supported when loop to be embedded is in fact embeddable */
+void ev_embed_start    (EV_P_ ev_embed *w);
+void ev_embed_stop     (EV_P_ ev_embed *w);
+void ev_embed_sweep    (EV_P_ ev_embed *w);
+# endif
+
 #endif
 
 #ifdef __cplusplus
